﻿//'*****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2007                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************/

// -------------------------------------------------------------------
// FVX Sample Program
// メインダイアログ
//
// 作成者 : R.Nagashima
//
// 履歴 : 2007/03/15 初版
// -------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
//FVXデータ構造の使用宣言
using FVDATASLib;

namespace Lsm
{
	public partial class frmMain : Form
	{
		public frmMain()
		{
			InitializeComponent();
		}
		
		//最大点群数
		private const int MAX_SETPOINTS = 4;

		//点群データを格納する配列
		private FvPointGroup[] m_PointGroup = new FvPointGroup[MAX_SETPOINTS];
		
		//点群毎の色の設定
		private Color[] m_PointGroupColor = new Color[MAX_SETPOINTS]; 

		//現在選択されている点群No
		private int m_intSelectNo = 0;

		//点群の図形ID格納配列w
		private int[] nIDGroup = new int[100];

		//作成中の点群数
		private int m_intCnt = 0;

		private void frmMain_Load(object sender, EventArgs e)
		{
			frmLicense fLicense = new frmLicense();
			FvPoint pPoint = new FvPoint();
			FvRectangle pRect = new FvRectangle();

			//変数の初期化
			for (int i = 0; i < MAX_SETPOINTS; i++)
			{
				m_PointGroup[i] = new FvPointGroup();
			}
			foreach (int i in nIDGroup)
			{
				nIDGroup[i] = 0;
			}

			//点群毎の色を設定
			m_PointGroupColor[0] = Color.Blue;		//点群１　青
			m_PointGroupColor[1] = Color.Lime;		//点群２　緑
			m_PointGroupColor[2] = Color.Yellow;	//点群３　黄
			m_PointGroupColor[3] = Color.Red;		//点群４　赤

			//使用許諾の確認
			if (fLicense.ShowDialog() == DialogResult.OK)
			{
				//画面の生成オーバーレイを点群分(4枚)+結果表示用(1枚)+FvxImage枠線表示用(1枚)生成
				axFvxDisplayEx1.CreateDisplay(MAX_SETPOINTS + 2, 0);
				axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());

				pRect.xs = 0;
				pRect.ys = 0;
				pRect.xe = axFvxImage1.ImageWidth - 1;
				pRect.ys = axFvxImage1.ImageHeight - 1;
				axFvxDisplayEx1.PenColor = Color.Gray;
				axFvxDisplayEx1.DrawData(4, pRect);//FvxImage外枠を描画

				axFvxDisplayEx1.RoiMode = FVXCOMDISPLAYEXLib.DISPLAYEX_ROI_MODE.DISPLAYEX_ROIDisable;
				axFvxDisplayEx1.RoiSetFigureMax(FVXCOMDISPLAYEXLib.DISPLAYEX_ROI_FIGURE_TYPE.DISPLAYEX_ROI_RECTANGLE, 1);
				axFvxDisplayEx1.RoiSetFigureMax(FVXCOMDISPLAYEXLib.DISPLAYEX_ROI_FIGURE_TYPE.DISPLAYEX_ROI_POINT, 100);

				//コンボボックスの設定
				cmbLSM.Items.Add("複数点群からの複数直線の算出");
				cmbLSM.Items.Add("複数点群からの複数同心円の算出");
				cmbLSM.Items.Add("点群からの長方形の算出");
				cmbLSM.Items.Add("点群からの比率固定長方形の算出");
				cmbLSM.SelectedIndex = 0;			
			}
			else
			{
				//終了
				Application.Exit();
				return;
			}			
		}

		//クリックされている間点群を生成
		private void chkICreatePoint_CheckedChanged(object sender, EventArgs e)
		{
			m_intSelectNo = cmbPointNo.SelectedIndex;

			if (chkICreatePoint.Checked)
			{
				fnPointsGet();
				axFvxDisplayEx1.RoiMode = FVXCOMDISPLAYEXLib.DISPLAYEX_ROI_MODE.DISPLAYEX_ROIEnable;
				axFvxDisplayEx1.RoiChangeActiveWithKey = true;
			}
			else
			{
				axFvxDisplayEx1.RoiChangeActiveWithKey = false;
				axFvxDisplayEx1.RoiMode = FVXCOMDISPLAYEXLib.DISPLAYEX_ROI_MODE.DISPLAYEX_ROIDisable;
				fnPointsRegist();
			}
		}

		//点群の変更
		private void cmbPointNo_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (chkICreatePoint.Checked)
			{
				fnPointsRegist();
				m_intSelectNo = cmbPointNo.SelectedIndex;
				fnPointsGet();

				axFvxDisplayEx1.RoiMode = FVXCOMDISPLAYEXLib.DISPLAYEX_ROI_MODE.DISPLAYEX_ROIEnable;
				axFvxDisplayEx1.RoiChangeActiveWithKey = true;
			}
		}

		//点群の消去
		private void btnClsPoint_Click(object sender, EventArgs e)
		{
			axFvxDisplayEx1.RoiDeleteAllFigures();

			for (int i = 0; i < MAX_SETPOINTS; i++)
			{
				m_PointGroup[i].Clear(-1);
				axFvxDisplayEx1.ClearOverlay(i);
			}
		}

		//結果の削除
		private void btnResultClear_Click(object sender, EventArgs e)
		{
			axFvxDisplayEx1.ClearOverlay(MAX_SETPOINTS);
		}

		//LSMの手法設定
		private void cmbLSM_SelectedIndexChanged(object sender, EventArgs e)
		{
			txtRatio.Enabled = false;
			txtLineAngle1.Enabled = false;
			txtLineAngle2.Enabled = false;
			txtLineAngle3.Enabled = false;
			txtLineAngle4.Enabled = false;
			txtWeight.Enabled = false;

			cmbPointNo.Items.Clear();
		
			switch (cmbLSM.SelectedIndex)
			{
				case 0: //複数点群からの複数直線の算出
					txtNum.Enabled = true;
					txtLineAngle1.Enabled = true;
					txtLineAngle2.Enabled = true;
					txtLineAngle3.Enabled = true;
					txtLineAngle4.Enabled = true;

					cmbPointNo.Items.Insert(0,"点群1 - 青");
					cmbPointNo.Items.Insert(1,"点群2 - 緑");
					cmbPointNo.Items.Insert(2,"点群3 - 黄");
					cmbPointNo.Items.Insert(3,"点群4 - 赤");
					cmbPointNo.SelectedIndex = 0;
					break;

				case 1: //複数点群からの複数同心円の算出
					txtNum.Enabled = true;
					cmbPointNo.Items.Insert(0,"点群1 - 青");
					cmbPointNo.Items.Insert(1,"点群2 - 緑");
					cmbPointNo.Items.Insert(2,"点群3 - 黄");
					cmbPointNo.Items.Insert(3,"点群4 - 赤");
					cmbPointNo.SelectedIndex = 0;
					break;

				case 2: //点群からの長方形の算出
					txtNum.Text = MAX_SETPOINTS.ToString();//点群は4固定
					txtNum.Enabled = false;

					cmbPointNo.Items.Insert(0,"点群1 - 上 - 青");
					cmbPointNo.Items.Insert(1,"点群2 - 右 - 緑");
					cmbPointNo.Items.Insert(2,"点群3 - 下 - 黄");
					cmbPointNo.Items.Insert(3,"点群4 - 左 - 赤");
					cmbPointNo.SelectedIndex = 0;
					break;

				case 3: //点群からの比率固定長方形の算出
					txtNum.Text = MAX_SETPOINTS.ToString();//点群は4固定
					txtNum.Enabled = false;
					txtRatio.Enabled = true;

					cmbPointNo.Items.Insert(0,"点群1 - 上 - 青");
					cmbPointNo.Items.Insert(1,"点群2 - 右 - 緑");
					cmbPointNo.Items.Insert(2,"点群3 - 下 - 黄");
					cmbPointNo.Items.Insert(3,"点群4 - 左 - 赤");
					cmbPointNo.SelectedIndex = 0;
					break;
			}
			//ロバスト推定のとき重み設定可
			if (rdoRobust1.Checked)
				txtWeight.Enabled = true;
		}

		private bool ApplyChanges(int intIndex, string txtLineangle, string txtWeight, bool blnRobust)
		{
			double dblTemp;

			if (txtLineangle != "IGNORE")
			{
				if(!(double.TryParse(txtLineangle,out dblTemp)) || dblTemp < -90 || dblTemp > 90)
				{
					MessageBox.Show("傾き角は -90°～　90°の範囲で指定して下さい","エラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
					return(false);
				}
				axFvxLSM1.set_LineAngle(intIndex,dblTemp);
			}
			if(blnRobust)
			{
				if(!(double.TryParse(txtWeight,out dblTemp)))
				{
					MessageBox.Show("値が不正です","エラー",MessageBoxButtons.OK,MessageBoxIcon.Error);
					return(false);
				}
				axFvxLSM1.set_Weight(intIndex,dblTemp);
			}
			return(true);
		}

		//実行
		private void btnExecute_Click(object sender, EventArgs e)
		{
			FvLineGroup pLineGroup = new FvLineGroup();
			FvCircleGroup pCircleGroup = new FvCircleGroup();
			FvPoint pPointCenter = new FvPoint();
			FvPoint pPointTopLeft = new FvPoint();
			FvPoint pPointTopRight = new FvPoint();
			FvPoint pPointBottomLeft = new FvPoint();
			FvPoint pPointBottomRight = new FvPoint();
			FvRectangle pRectangle = new FvRectangle();
			int intErr = 0;
			int intTemp = 0;
			double dblTemp = 0;
			bool blnRobust = false;
			double dblTime = 0;
			double dblAngle = 0;
			double dblRatio = 0;
			int i = 0;

			//初期化
			chkICreatePoint.Checked = false;
			axFvxDisplayEx1.ClearOverlay(MAX_SETPOINTS);
			txtResult.Text = "";
			axFvxImage1.ClearImage();

			try
			{
				//入力チェック
				if (int.TryParse(txtNum.Text, out intTemp) == false || intTemp > MAX_SETPOINTS)
				{
					MessageBox.Show("このサンプルプログラムでは点群数は1～4までしか設定できません", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
					return;
				}
				axFvxLSM1.Num = intTemp;

				//点群数と点群データの数の確認
				for (i = 0; i < axFvxLSM1.Num; i++)
				{
					if (m_PointGroup[i].Count == 0)
					{
						MessageBox.Show("点群数分の点群を作成して下さい", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
						return;
					}
				}

				//ロバスト推定するかどうか
				if (rdoRobust1.Checked == true) blnRobust = true;

				switch (cmbLSM.SelectedIndex)
				{
					case 0://複数点群からの複数直線の算出
						i = 0;
						if (axFvxLSM1.Num > i)
						{
							if (ApplyChanges(0, txtLineAngle1.Text, txtWeight.Text, blnRobust) == false)
								return;
							i++;
						}
						if (axFvxLSM1.Num > i)
						{
							if (ApplyChanges(1, txtLineAngle2.Text, txtWeight.Text, blnRobust) == false)
								return;
							i++;
						}
						if (axFvxLSM1.Num > i)
						{
							if (ApplyChanges(2, txtLineAngle3.Text, txtWeight.Text, blnRobust) == false)
								return;
							i++;
						}
						if (axFvxLSM1.Num > i)
						{
							if (ApplyChanges(3, txtLineAngle4.Text, txtWeight.Text, blnRobust) == false)
								return;
							i++;
						}

						for (i = 0; i < axFvxLSM1.Num; i++)
							axFvxLSM1.SetPoints(i, m_PointGroup[i]);

						//実行
						axFvxPerformanceCounter1.StartCount();
						axFvxLSM1.CalcLines(pLineGroup, blnRobust);
						axFvxPerformanceCounter1.StopCount(out dblTime);
						//結果出力
						for (i = 0; i < pLineGroup.Count; i++)
						{
							axFvxDisplayEx1.PenColor = m_PointGroupColor[i];
							axFvxDisplayEx1.DrawData(MAX_SETPOINTS, pLineGroup.Item2(i));
							txtResult.Text = txtResult.Text + string.Format("直線 {0} a = {1}  b = {2}  c = {3}\r\n", i + 1, pLineGroup.Item2(i).a, pLineGroup.Item2(i).b, pLineGroup.Item2(i).c);

						}
						break;


					case 1://複数点群からの複数同心円の算出
						for (i = 0; i < axFvxLSM1.Num; i++)
						{
							if (ApplyChanges(i, "IGNORE", txtWeight.Text, blnRobust) == false)
								return;
							axFvxLSM1.SetPoints(i, m_PointGroup[i]);
						}

						//実行
						axFvxPerformanceCounter1.StartCount();
						axFvxLSM1.CalcCircles(pCircleGroup, blnRobust);
						axFvxPerformanceCounter1.StopCount(out dblTime);

						//結果の出力
						for (i = 0; i < pCircleGroup.Count; i++)
						{
							axFvxDisplayEx1.PenColor = m_PointGroupColor[i];
							axFvxDisplayEx1.DrawData(MAX_SETPOINTS, pCircleGroup.Item(i));
							txtResult.Text = txtResult.Text + string.Format("同心円 {0} x = {1}  y = {2}  r = {3}\r\n", i + 1, pCircleGroup.Item2(i).x, pCircleGroup.Item2(i).y, pCircleGroup.Item2(i).radius);
						}
						break;

					case 2: //点群からの長方形の算出
						for (i = 0; i < MAX_SETPOINTS; i++)
						{
							if (ApplyChanges(i, "IGNORE", txtWeight.Text, blnRobust) == false)
								return;
							axFvxLSM1.SetPoints(i, m_PointGroup[i]);
						}

						//実行
						axFvxPerformanceCounter1.StartCount();
						axFvxLSM1.CalcRectangle(pPointCenter, pPointTopLeft, pPointTopRight, pPointBottomLeft, pPointBottomRight, out dblAngle, blnRobust);
						axFvxPerformanceCounter1.StopCount(out dblTime);

						//FvRectangleに格納しデータを表示
						pRectangle.Set4Points(pPointTopLeft, pPointTopRight, pPointBottomRight, pPointBottomLeft);
						axFvxDisplayEx1.PenColor = Color.Magenta;
						axFvxDisplayEx1.DrawData(MAX_SETPOINTS, pPointCenter);
						axFvxDisplayEx1.DrawData(MAX_SETPOINTS, pRectangle);

						txtResult.Text = string.Format("長方形の中心座標 = ( {0:0.0} , {1:0.0} )\r\n" +
														"長方形の左上座標 = ( {2:0.0} , {3:0.0} )\r\n" +
														"長方形の右上座標 = ( {4:0.0} , {5:0.0} )\r\n" +
														"長方形の左下座標 = ( {6:0.0} , {7:0.0} )\r\n" +
														"長方形の右下座標 = ( {8:0.0} , {9:0.0} )\r\n",
														pPointCenter.x, 
														pPointCenter.y,
														pPointTopLeft.x,
														pPointTopLeft.y,
														pPointTopRight.x,
														pPointTopRight.y,
														pPointBottomLeft.x,
														pPointBottomLeft.y,
														pPointBottomRight.x,
														pPointBottomRight.y);

						break;

					case 3: //点群からの比率固定長方形の算出
						for (i = 0; i < MAX_SETPOINTS; i++)
						{
							if (ApplyChanges(i, "IGNORE", txtWeight.Text, blnRobust) == false)
								return;
							axFvxLSM1.SetPoints(i, m_PointGroup[i]);
						}

						//長方形の縦横比(縦/横)
						if (double.TryParse(txtRatio.Text, out dblTemp) == false)
						{
							MessageBox.Show("縦横比が不正です", "エラー", MessageBoxButtons.OK, MessageBoxIcon.Error);
							return;
						}
						dblRatio = dblTemp;

						//実行
						axFvxPerformanceCounter1.StartCount();
						axFvxLSM1.CalcRectangle2(dblRatio, pPointCenter, pPointTopLeft, pPointTopRight, pPointBottomLeft, pPointBottomRight, out dblAngle, blnRobust);
						axFvxPerformanceCounter1.StopCount(out dblTime);

						//FvRectangelに格納しデータを表示
						pRectangle.Set4Points(pPointTopLeft, pPointTopRight, pPointBottomRight, pPointBottomLeft);
						axFvxDisplayEx1.PenColor = Color.Magenta;
						axFvxDisplayEx1.DrawData(MAX_SETPOINTS, pPointCenter);
						axFvxDisplayEx1.DrawData(MAX_SETPOINTS, pRectangle);

						txtResult.Text = string.Format("長方形の中心座標 = ( {0:0.0} , {1:0.0} )\r\n" +
														"長方形の左上座標 = ( {2:0.0} , {3:0.0} )\r\n" +
														"長方形の右上座標 = ( {4:0.0} , {5:0.0} )\r\n" +
														"長方形の左下座標 = ( {6:0.0} , {7:0.0} )\r\n" +
														"長方形の右下座標 = ( {8:0.0} , {9:0.0} )\r\n",
														pPointCenter.x, pPointCenter.y,
														pPointTopLeft.x, pPointTopLeft.y,
														pPointTopRight.x, pPointTopRight.y,
														pPointBottomLeft.x, pPointBottomLeft.y,
														pPointBottomRight.x, pPointBottomRight.y);
						break;
				}
				//結果画像の表示
				axFvxDisplayEx1.DrawImage(axFvxImage1.GetOcx());
				//計測時間の出力
				txtExecuteTime.Text = dblTime.ToString("0.00ms");
			}
			catch (System.Runtime.InteropServices.COMException)
			{
				axFvxLSM1.GetErrReport(out intErr);
				MessageBox.Show(GetChangeImageErrMesg(intErr), "エラー発生", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.ToString());
			}
		}

		//点の作成
		private void axFvxDisplayEx1_RoiNewFigure(object sender, AxFVXCOMDISPLAYEXLib._IFvxDisplayExEvents_RoiNewFigureEvent e)
		{
			//カレント点群IDを取得する
			m_intSelectNo = cmbPointNo.SelectedIndex;

			//図形IDを配列に格納する
			nIDGroup[m_intCnt] = e.nFigID;

			//図係数カウントアップ
			m_intCnt = m_intCnt + 1;
		}

		//点の削除
		private void axFvxDisplayEx1_RoiDeleteFigureEvent(object sender, AxFVXCOMDISPLAYEXLib._IFvxDisplayExEvents_RoiDeleteFigureEvent e)
		{
			//カレント点群のIDを取得する
			m_intSelectNo = cmbPointNo.SelectedIndex;

			//対象点のIDを削除する
			for (int i = 0; i < m_intCnt; i++)
			{
				if (nIDGroup[i] == e.nFigID)
				{
					for (int j = 0; j < m_intCnt; j++)
					{
						nIDGroup[j] = nIDGroup[j + 1];
					}
				}
			}
			//図形カウントダウン
			m_intCnt = m_intCnt - 1;
		}

		//ロバスト推定を行うかどうか
		private void rdoRobust0_CheckedChanged(object sender, EventArgs e)
		{
			if (rdoRobust0.Checked == true)
				txtWeight.Enabled = false;
			else txtWeight.Enabled = true;
		}

		//設定された点群を取得しROIで図形を設定
		private void fnPointsGet()
		{
			FvPoint pPoint = new FvPoint();
			int intGroupCnt = 0;
			
			intGroupCnt = m_PointGroup[m_intSelectNo].Count;
			for (int i = 0; i < intGroupCnt; i++)
			{
				//m_PointGroup[i].Add(pPoint, -1);
				pPoint.x = m_PointGroup[m_intSelectNo].Item2(i).x;
				pPoint.y = m_PointGroup[m_intSelectNo].Item2(i).y;
				axFvxDisplayEx1.RoiCreateFigure(pPoint, true);
			}
		}

		//設定された点群を構造体に保存しオーバーレイに表示
		private void fnPointsRegist()
		{
			FvPoint pPoint = new FvPoint();
			int intFigureNum = 0;

			intFigureNum = axFvxDisplayEx1.RoiGetFigureNum(FVXCOMDISPLAYEXLib.DISPLAYEX_ROI_FIGURE_TYPE.DISPLAYEX_ROI_POINT);
			 //一旦pPoint全て削除
			axFvxDisplayEx1.ClearOverlay(m_intSelectNo);

			if(intFigureNum > 0)
			{
				m_PointGroup[m_intSelectNo].Clear(-1);
				for (int i = 0; i < intFigureNum; i++)
				{
					axFvxDisplayEx1.RoiGetFigure(nIDGroup[i],pPoint );
					m_PointGroup[m_intSelectNo].Add(pPoint,-1);
					axFvxDisplayEx1.PenColor = m_PointGroupColor[m_intSelectNo];
					axFvxDisplayEx1.DrawData(m_intSelectNo,pPoint);
				}
			}

			//現在描画されているROIを削除し、オーバーレイに書き込み
			axFvxDisplayEx1.RoiDeleteAllFigures();
			m_intCnt = 0;
		}

		private string GetChangeImageErrMesg(int intErr)
		{
			string Str = "";
			switch (intErr)
			{
				case 8805:
					Str = "パラメータが不正です";
					break;
				case 8806:
					Str = "引数の型が違います";
					break;
				case 8807:
					Str = "引数にNULLが指定されています";
					break;
				case 8810:
					Str = "メモリ確保に失敗しました";
					break;
				case 8811:
					Str = "計算不可能です";
					break;
				case 8812:
					Str = "点が少なすぎます";
					break;
				case 8813:
					Str = "Numプロパティが0です";
					break;
			}
			return (Str);
		}

		private void txtNum_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '4') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtRatio_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtLineAngle1_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '-' && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtLineAngle2_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '-' && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtLineAngle3_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '-' && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtLineAngle4_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '-' && e.KeyChar != '\b')
				e.Handled = true;
		}
		private void txtWeight_KeyPress(object sender, KeyPressEventArgs e)
		{
			if ((e.KeyChar < '0' || e.KeyChar > '9') && e.KeyChar != '\b')
				e.Handled = true;
		}

		private void txtNum_TextChanged(object sender, EventArgs e)
		{
			int Temp;
			int.TryParse(txtNum.Text,out Temp);
			switch (Temp)
			{
				case 0:
					txtLineAngle1.Enabled = false;
					txtLineAngle2.Enabled = false;
					txtLineAngle3.Enabled = false;
					txtLineAngle4.Enabled = false;
					break;
				case 1:
					txtLineAngle1.Enabled = true;
					txtLineAngle2.Enabled = false;
					txtLineAngle3.Enabled = false;
					txtLineAngle4.Enabled = false;
					break;
				case 2:
					txtLineAngle1.Enabled = true;
					txtLineAngle2.Enabled = true;
					txtLineAngle3.Enabled = false;
					txtLineAngle4.Enabled = false;
					break;
				case 3:
					txtLineAngle1.Enabled = true;
					txtLineAngle2.Enabled = true;
					txtLineAngle3.Enabled = true;
					txtLineAngle4.Enabled = false;
					break;
				case 4:
					txtLineAngle1.Enabled = true;
					txtLineAngle2.Enabled = true;
					txtLineAngle3.Enabled = true;
					txtLineAngle4.Enabled = true;
					break;
			}
		}
	}
}